#include "main.h"
#include "Manipulator.h"
#include "hslee_delay.h"
#include "hslee_uart.h"
#include "hslee_i2c_master.h"
#include "hslee_spi.h"
#include "hslee_adc.h"
#include "hslee_timer.h"

#pragma config FPLLMUL  = MUL_20        // PLL Multiplier
#pragma config FPLLIDIV = DIV_5         // PLL Input Divider
#pragma config FPLLODIV = DIV_1         // PLL Output Divider
#pragma config FPBDIV   = DIV_1         // Peripheral Clock divisor
#pragma config FWDTEN   = OFF           // Watchdog Timer 
#pragma config WDTPS    = PS1           // Watchdog Timer Postscale
#pragma config FCKSM    = CSDCMD        // Clock Switching & Fail Safe Clock Monitor
#pragma config OSCIOFNC = OFF           // CLKO Enable
#pragma config POSCMOD  = HS            // Primary Oscillator
#pragma config IESO     = OFF           // Internal/External Switch-over
#pragma config FSOSCEN  = OFF           // Secondary Oscillator Enable
#pragma config FNOSC    = PRIPLL        // Oscillator Selection
#pragma config CP       = OFF           // Code Protect
#pragma config BWP      = OFF           // Boot Flash Write Protect
#pragma config PWP      = OFF           // Program Flash Write Protect
#pragma config ICESEL   = ICS_PGx2      // ICE/ICD Comm Channel Select
#pragma config DEBUG    = OFF           // Debugger Disabled for Starter Kit

UINT8  uart1_rx;
UINT8  uart2_rx;

char zzz[250];
char protocol[255];

int main(void)
{
    char   zzz_main[250];
    SYSTEMConfigPerformance(SYS_FREQ);
    
    Init_Uart1(115200);
    Init_Uart2(115200);

    mPORTESetPinsDigitalOut( BIT_4 );//LED

    mPORTDSetPinsDigitalOut( BIT_6 );//UART2_EN
    mPORTDSetPinsDigitalOut( BIT_7 );//UART1_EN
	
    LED = 1;

    UART1_EN = 0;
    UART2_DI;
	
    INTEnableSystemMultiVectoredInt();   // Interrupt vectoring
    Delayms(3000);
	
    while(1)
    {
         LED = ~LED;
    }
    return 0;
}

INT8 u1_start = FALSE;
INT8 u1_cmd0 = 0;
INT8 u1_cmd1 = 0;
UINT8 u1_data_len = 0;
char u1_data[255];

void UART1_Protocol_Parsing(INT8 cmd0, INT8 cmd1, char * data)
{
    UINT8 i=0;
    //sprintf(zzz,"CMD0:%c \t CMD1:%c \t DATA:%s \r\n",cmd0, cmd1, data);TxString1(zzz);

    switch(cmd0)
    {
        case 'P': //Platform
        {
            switch(cmd1)
            {
                //<Move Driving Motor
                // [~PM100|100`]
                case 'M':
                case 'O': //not return encoder value
                {
                    char left[10] = {0,};	//Left rpm var
                    char right[10] = {0,};	//Right rpm var
                    UINT8 token = 0;
                    UINT8 token_cnt = 0;

                    for(i=0;i<strlen(data);i++)
                    {
                        if(data[i] == '|')
                        {
                            token++;
                            token_cnt = i+1;
                        }
                        else
                        {
                            if(token==0)
                                left[i] = data[i];
                            else if(token == 1)
                                right[i-token_cnt] = data[i];
                        }
                    }
                    if(token != 0)
                    {
                        if(cmd1 == 'M')
                            sprintf(protocol,"MP%s|%s`",left,right);
                        else if(cmd1 == 'O')
                            sprintf(protocol,"MO%s|%s`",left,right);
                        SendData_Platform(protocol);
                    }
                    break;
                }
                //Position Tilt Angle
                // [~PT100|100`]
                case 'T': //Tilt
                {
                    char front[10] = {0,};	//front angle
                    char back[10] = {0,};	//back angle
                    UINT8 token = 0;
                    UINT8 token_cnt = 0;

                    for(i=0;i<strlen(data);i++)
                    {
                        if(data[i] == '|')
                        {
                            token++;
                            token_cnt = i+1;
                        }
                        else
                        {
                            if(token==0)
                                front[i] = data[i];
                            else if(token == 1)
                                back[i-token_cnt] = data[i];
                        }
                    }
                    if(token != 0)
                    {
                        sprintf(protocol,"MT%s|%s`",front,back);
                        SendData_Platform(protocol);
                    }
                    break;
                }
            }
        break;
        }
        case 'M': //Manipualtor Move Angle Joint
        {
            switch(cmd1)
            {
                //Move Joint
                // [~M010`]
                case '0':
                case '1':
                case '2':
                case '3':
                case '4':
                case '5':
                {
                    char m_angle[10] = {0,};	//Left rpm var
                    char m_rpm[10] = {0,};	//Right rpm var
                    UINT8 token = 0;
                    UINT8 token_cnt = 0;

                    for(i=0;i<strlen(data);i++)
                    {
                        if(data[i] == '|')
                        {
                            token++;
                            token_cnt = i+1;
                        }
                        else
                        {
                            if(token==0)
                                m_angle[i] = data[i];
                            else if(token == 1)
                                m_rpm[i-token_cnt] = data[i];
                        }
                    }
                    if(token != 0)
                    {
                        sprintf(protocol,"T%c%s|%s`",cmd1,m_angle,m_rpm);
                        SendData_Manipulator(protocol);
                    }
                    break;
                }
                //Velocity Move -> �׸��۴� ���ܵȴ�...
                case 'a':
                case 'b':
                case 'c':
                case 'd':
                case 'e':
                case 'f':
                {
                    if(cmd1 == 'a')		cmd1 = '0';
                    else if(cmd1 == 'b')cmd1 = '1';
                    else if(cmd1 == 'c')cmd1 = '2';
                    else if(cmd1 == 'd')cmd1 = '3';
                    else if(cmd1 == 'e')cmd1 = '4';
                    else if(cmd1 == 'f')cmd1 = '5';

                    sprintf(protocol,"V%c%s`",cmd1,data);
                    SendData_Manipulator(protocol);
                    break;
                }
                //STOP
                case 'A':
                case 'B':
                case 'C':
                case 'D':
                case 'E':
                case 'F':
                case 'G':
                {
                    if(cmd1 == 'A')		cmd1 = '0';
                    else if(cmd1 == 'B')cmd1 = '1';
                    else if(cmd1 == 'C')cmd1 = '2';
                    else if(cmd1 == 'D')cmd1 = '3';
                    else if(cmd1 == 'E')cmd1 = '4';
                    else if(cmd1 == 'F')cmd1 = '5';
                    else if(cmd1 == 'G')cmd1 = 'A';
                    sprintf(protocol,"S%c`",cmd1);
                    SendData_Manipulator(protocol);
                    break;
                }
                case '?':
                {
                    SendData_Manipulator("T?`");
                    break;
                }
                //Enalbe get
                case '/':
                {
                    SendData_Manipulator("E?`");
                    break;
                }
                //Limit Set
                case 'L':
                {
                    if(data == "0")
                        SendData_Manipulator("L0`");
                    else if(data == "1")
                        SendData_Manipulator("L1`");
                    break;
                }
                case 'I':
                {
                    SendData_Manipulator("I0`");
                    break;
                }
            }
            break;
        }
    }
}



void __ISR(_UART1_VECTOR, ipl5) _uart1(void) 
{
    if(INTGetFlag(INT_SOURCE_UART_RX(UART1)))
    {
            INTClearFlag(INT_SOURCE_UART_RX(UART1));
            uart1_rx = UARTGetDataByte(UART1);

            if(u1_start == TRUE)
            {
                if(uart1_rx == '`')//ETX
                {
                    u1_data[u1_data_len-2] = '\0';
                    UART1_Protocol_Parsing(u1_cmd0, u1_cmd1,u1_data);

                    u1_cmd0 = 0;
                    u1_cmd1 = 0;
                    u1_data_len = 0;
                    u1_start = FALSE;
                }
                else
                {
                    if(u1_data_len == 0)
                        u1_cmd0 = uart1_rx;
                    else if(u1_data_len == 1)
                        u1_cmd1 = uart1_rx;
                    else
                        u1_data[u1_data_len-2] = uart1_rx;
                    u1_data_len++;
                }
        }
        if(uart1_rx == '~' && u1_start == FALSE)//StartFlag
            u1_start = TRUE;
    }
    U1STAbits.OERR=0;
}	





void __ISR(_UART2_VECTOR, ipl3) _uart2(void) 	
{
    if(INTGetFlag(INT_SOURCE_UART_RX(UART2)))
    {
        INTClearFlag(INT_SOURCE_UART_RX(UART2));
        uart2_rx = UARTGetDataByte(UART2);
        TxData1(uart2_rx);
    }
    U2STAbits.OERR=0;
}



void SendData_Platform(char* data)
{
    //data = ~ 0x01 MP 100|100`
    // MP100|100`
    UINT8 len = strlen(data);
    UINT8 	tx_len = len + 2;
    UINT8   tx_buf[255] = {0,};
    UINT8 	i;

    tx_buf[0] = '~';
    tx_buf[1] = 0x01;
    for(i=2;i<tx_len;i++)
            tx_buf[i] = data[i-2];
    UART2_EN;//Tx Enable ON;
    for(i=0;i<tx_len;i++)
            TxData2(tx_buf[i]);
    UART2_DI;
	//Delayms(10);//��Ȯ�Ѱ��� �ƴϴ�.. Ʃ���ʿ�..
}


void SendData_Manipulator(char* data)
{
    //data = ~ 0x01 MP 100|100`
    // MP100|100`
    UINT8 len = strlen(data);
    UINT8 	tx_len = len + 2;
    UINT8   tx_buf[255] = {0,};
    UINT8 	i;

    tx_buf[0] = '~';
    tx_buf[1] = 0x02;
    for(i=2;i<tx_len;i++)
            tx_buf[i] = data[i-2];
    UART2_EN;//Tx Enable ON;
    for(i=0;i<tx_len;i++)
        TxData2(tx_buf[i]);
    UART2_DI;
    //Delayms(10);//��Ȯ�Ѱ��� �ƴϴ�.. Ʃ���ʿ�..
}

